﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.IO;
using System.Linq;
using System.Xml;
using System.Xml.Linq;

namespace SeaVan
{
    public class CrossingXmlParser
    {

        #region Fields

        private static short BC_CROSSING_COUNT = 4;

        #endregion


        #region ParseXmlUs (OLD)

        //internal static void ParseXmlUs(Stream result)
        //{
        //    //XDocument xdoc = new XDocument();
        //    XDocument xdoc = XDocument.Load(result);

        //    // Get the date/time the WSDOT data was last updated (already in PST/PDT).
        //    XElement guidElement = xdoc.Descendants("guid").First();
        //    if (guidElement != null)
        //    {
        //        string guidString = guidElement.Value;
        //        // "Fri+Mar+30+14%3a15%3a22+PDT+2012"

        //        guidString = guidString.Replace('+', ' ');
        //        guidString = guidString.Replace("%3a", ":");
        //        // "Fri Mar 30 14:15:22 PDT 2012"

        //        string[] dateParts = guidString.Split(' ');
        //        string updateString = String.Format("{0}, {1} {2} {3} {4}",
        //            dateParts[0], dateParts[2], dateParts[1], dateParts[5], dateParts[3]);
        //        // "Fri, 30 Mar 2012 14:15:22"

        //        DateTime date;
        //        if (DateTime.TryParse(updateString, out date))
        //        {
        //            App.Crossings.LastUpdateUs = "last update (sea) - " + date;
        //            Debug.WriteLine(String.Format("webClientUs_OpenReadCompleted: {0} [parse ok]", App.Crossings.LastUpdateUs));
        //        }
        //        else
        //        {
        //            App.Crossings.LastUpdateUs = "last update (sea) - " + DateTime.Now;
        //            Debug.WriteLine(String.Format("webClientUs_OpenReadCompleted: {0} [defaulted]", App.Crossings.LastUpdateUs));
        //        }
        //    }
        //    else
        //    {
        //        App.Crossings.LastUpdateUs = "last update (sea) - " + DateTime.Now;
        //        Debug.WriteLine(String.Format("webClientUs_OpenReadCompleted: [guid element not found]"));
        //    }

        //    // Extract the CDATA section as a string, and reduce it to an array of crossing|wait-time pairs.
        //    XNode node = xdoc.DescendantNodes().Single(el => el.NodeType == XmlNodeType.CDATA);
        //    string content = node.Parent.Value;
        //    GetWaitTimesUs(content);
        //}

        #endregion


        #region ParseXmlUs

        //   <title>Canadian Border Crossings - Fri Jun 15 11:15:00 PDT 2012</title> 

        internal static void ParseXmlUs(Stream result)
        {
            XDocument xdoc = XDocument.Load(result);

            // Get the date/time the WSDOT data was last updated (already in PST/PDT).
            XElement itemElement = xdoc.Descendants("item").First();
            XElement titleElement = itemElement.Descendants("title").First();
            if (titleElement != null)
            {
                string titleString = titleElement.Value;
                // "Canadian Border Crossings - Fri Jun 15 11:15:00 PDT 2012"

                titleString = titleString.Replace("Canadian Border Crossings - ", "");
                // "Fri Jun 15 11:15:00 PDT 2012"

                string[] dateParts = titleString.Split(' ');
                string updateString = String.Format("{0}, {1} {2} {3} {4}",
                    dateParts[0], dateParts[2], dateParts[1], dateParts[5], dateParts[3]);
                // "Fri, 15 Jun 2012 11:15:00"

                DateTime date;
                if (DateTime.TryParse(updateString, out date))
                {
                    App.Crossings.LastUpdateUs = "last update (sea) - " + date;
                    Debug.WriteLine(String.Format("webClientUs_OpenReadCompleted: {0} [parse ok]", App.Crossings.LastUpdateUs));
                }
                else
                {
                    App.Crossings.LastUpdateUs = "last update (sea) - " + DateTime.Now;
                    Debug.WriteLine(String.Format("webClientUs_OpenReadCompleted: {0} [defaulted]", App.Crossings.LastUpdateUs));
                }
            }
            else
            {
                App.Crossings.LastUpdateUs = "last update (sea) - " + DateTime.Now;
                Debug.WriteLine(String.Format("webClientUs_OpenReadCompleted: [title element not found]"));
            }

            // Extract the CDATA section as a string, and reduce it to an array of crossing|wait-time pairs.
            XNode node = xdoc.DescendantNodes().Single(el => el.NodeType == XmlNodeType.CDATA);
            string content = node.Parent.Value;
            GetWaitTimesUs(content);
        }

        #endregion


        #region GetWaitTimesUs

        private static void GetWaitTimesUs(string content)
        {
            content = content.ToLower();
            content = content.Replace("<br/>", "");
            content = content.Replace("<strong>station name:</strong>", ",");
            content = content.Replace("<strong>wait time:</strong>", "|");

            string[] crossingData = content.Split(',');
            Dictionary<string, string> usCrossings = new Dictionary<string, string>();
            for (int i = 1; i < crossingData.Length; i++)
            {
                string[] crossingParts = crossingData[i].Split('|');
                usCrossings.Add(crossingParts[0], crossingParts[1]);
            }

            foreach (BorderCrossing crossing in App.Crossings.Crossings)
            {
                switch (crossing.PortID)
                {
                    case "300401":
                        crossing.PassengerDelayTimeUs = GetValueOrDefault(usCrossings, "sr543");
                        crossing.NexusDelayTimeUs = GetValueOrDefault(usCrossings, "sr543nexus");
                        crossing.CommercialDelayTimeUs = GetValueOrDefault(usCrossings, "sr543trucks");
                        crossing.FastDelayTimeUs = GetValueOrDefault(usCrossings, "sr543trucksfast");
                        Debug.WriteLine(String.Format("GetWaitTimesUsa: 300401: std={0},nex={1},com={2},fst={3}",
                            crossing.PassengerDelayTimeUs, crossing.NexusDelayTimeUs, crossing.CommercialDelayTimeUs, crossing.FastDelayTimeUs));
                        break;
                    case "300402":
                        crossing.PassengerDelayTimeUs = GetValueOrDefault(usCrossings, "i5");
                        crossing.NexusDelayTimeUs = GetValueOrDefault(usCrossings, "i5nexus");
                        Debug.WriteLine(String.Format("GetWaitTimesUsa: 300402: std={0},nex={1}",
                            crossing.PassengerDelayTimeUs, crossing.NexusDelayTimeUs));
                        break;
                    case "300901":
                        crossing.PassengerDelayTimeUs = GetValueOrDefault(usCrossings, "sr9");
                        Debug.WriteLine(String.Format("GetWaitTimesUsa: 300901: std={0}", crossing.PassengerDelayTimeUs));
                        break;
                    case "302301":
                        crossing.PassengerDelayTimeUs = GetValueOrDefault(usCrossings, "sr539");
                        Debug.WriteLine(String.Format("GetWaitTimesUsa: 302301: std={0}", crossing.PassengerDelayTimeUs));
                        break;
                }
            }
        }

        private static string GetValueOrDefault(Dictionary<string, string> d, string key)
        {
            string s;
            d.TryGetValue(key, out s);
            if (s == null)
            {
                s = "no data";
            }
            return s;
        }

        #endregion


        #region ParseXmlCa

        internal static void ParseXmlCa(Stream result)
        {
            //XDocument xdoc = new XDocument();
            XDocument xdoc = XDocument.Load(result);
            var waitTimes = xdoc.Elements("border_wait_time");

            // Get the date/time the CBP data was last updated, and convert from EST/EDT to PST/PDT.
            string d = string.Empty;
            string t = string.Empty;
            XElement dateElement = waitTimes.Elements("last_updated_date").First();
            if (dateElement != null)
            {
                d = dateElement.Value;
            }
            XElement timeElement = waitTimes.Elements("last_updated_time").First();
            if (timeElement != null)
            {
                t = timeElement.Value;
            }
            if (!string.IsNullOrEmpty(d) && !string.IsNullOrEmpty(t))
            {
                DateTime date;
                if (DateTime.TryParse(d + " " + t + "-4:00", out date))
                {
                    App.Crossings.LastUpdateCa = "last update (van) - " + date;
                    Debug.WriteLine(String.Format("webClientCa_OpenReadCompleted: {0} [parse ok]", App.Crossings.LastUpdateCa));
                }
                else
                {
                    App.Crossings.LastUpdateCa = "last update (van) - " + DateTime.Now;
                    Debug.WriteLine(String.Format("webClientCa_OpenReadCompleted: {0} [defaulted]", App.Crossings.LastUpdateCa));
                }
            }
            else
            {
                App.Crossings.LastUpdateCa = "last update (van) - " + DateTime.Now;
                Debug.WriteLine(String.Format("webClientCa_OpenReadCompleted: {0} [date/time not found]", App.Crossings.LastUpdateCa));
            }

            // Out of the 72 ports of entry, add the 4 BC ports we care about to the list.
            short count = 0;
            foreach (XElement port in waitTimes.Elements("port"))
            {
                string portNumber = port.Element("port_number").Value;
                BorderCrossing crossing = App.Crossings[portNumber];
                if (crossing != null)
                {
                    GetWaitTimesCa(port, crossing);
                    if (++count > BC_CROSSING_COUNT)
                    {
                        break;
                    }
                }
            }
        }

        #endregion


        #region GetWaitTimesCa

        private static void GetWaitTimesCa(XElement port, BorderCrossing crossing)
        {
            XElement passenger = port.Element("passenger_vehicle_lanes");
            if (passenger != null)
            {
                XElement passengerStandard = passenger.Element("standard_lanes");
                if (passengerStandard != null)
                {
                    crossing.PassengerDelayTimeCa = GetLaneDataCa(passengerStandard);
                }
                else
                {
                    crossing.PassengerDelayTimeCa = "no data";
                }

                XElement passengerNexus = passenger.Element("NEXUS_SENTRI_lanes");
                if (passengerNexus != null)
                {
                    crossing.NexusDelayTimeCa = GetLaneDataCa(passengerNexus);
                }
                else
                {
                    crossing.NexusDelayTimeCa = "no data";
                }

                XElement passengerReady = passenger.Element("ready_lanes");
                if (passengerReady != null)
                {
                    crossing.ReadyDelayTimeCa = GetLaneDataCa(passengerReady);
                }
                else
                {
                    crossing.ReadyDelayTimeCa = "no data";
                }
            }
            else
            {
                crossing.PassengerDelayTimeCa = "no data";
                crossing.NexusDelayTimeCa = "no data";
                crossing.ReadyDelayTimeCa = "no data";
            }
            Debug.WriteLine(String.Format("GetWaitTimesCanada: std={0},nex={1},rdy={2}",
                crossing.PassengerDelayTimeCa, crossing.NexusDelayTimeCa, crossing.ReadyDelayTimeCa));

            XElement commercial = port.Element("commercial_vehicle_lanes");
            if (commercial != null)
            {
                XElement commercialStandard = commercial.Element("standard_lanes");
                if (commercialStandard != null)
                {
                    crossing.CommercialDelayTimeCa = GetLaneDataCa(commercialStandard);
                }
                else
                {
                    crossing.CommercialDelayTimeCa = "no data";
                }

                XElement commercialFast = commercial.Element("FAST_lanes");
                if (commercialFast != null)
                {
                    crossing.FastDelayTimeCa = GetLaneDataCa(commercialFast);
                }
                else
                {
                    crossing.FastDelayTimeCa = "no data";
                }
            }
            else
            {
                crossing.CommercialDelayTimeCa = "no data";
                crossing.FastDelayTimeCa = "no data";
            }
            Debug.WriteLine(String.Format("GetWaitTimesCanada: com={0},fst={1}",
                crossing.CommercialDelayTimeCa, crossing.FastDelayTimeCa));
        }

        #endregion


        #region GetLaneDataCa

        private static string GetLaneDataCa(XElement laneElement)
        {
            string waitTime = "no data";
            XElement statusElement = laneElement.Element("operational_status");
            if (statusElement != null)
            {
                string status = statusElement.Value.Trim().ToLower();
                if (status == "delay")
                {
                    XElement time = laneElement.Element("delay_minutes");
                    if (time != null)
                    {
                        waitTime = time.Value.Replace("min", "").Trim();
                    }
                    else
                    {
                        waitTime = "delay";
                    }
                }
                else if (status == "no delay")
                {
                    waitTime = "0";
                }
                else if (status == "n/a")
                {
                    waitTime = "-";
                }
                else if (status == "lanes closed")
                {
                    waitTime = "closed";
                }
                else if (status == "update pending")
                {
                    waitTime = "updating";
                }
                else
                {
                    waitTime = status;
                }
            }
            return waitTime;
        }

        #endregion

    }
}
