﻿using System;
using System.Diagnostics;
using System.IO.IsolatedStorage;

namespace SeaVan
{
    public class AppSettings
    {

        #region Fields

        private IsolatedStorageSettings isolatedStore;

        // The isolated storage key names of our settings
        private const string LocationSettingKeyName = "LocationSetting";
        private const string AutoRefreshSettingKeyName = "AutoRefreshSetting";

        // The default value of our settings
        private const bool LocationSettingDefault = true;
        private const bool AutoRefreshSettingDefault = true;

        #endregion


        #region Init

        public AppSettings()
        {
            try
            {
                isolatedStore = IsolatedStorageSettings.ApplicationSettings;
            }
            catch (Exception ex)
            {
                Debug.WriteLine(ex.ToString());
            }
        }

        // Update a setting value for our application. If the setting does not exist, then add the setting.
        private bool AddOrUpdateValue(string Key, Object value)
        {
            bool valueChanged = false;

            if (isolatedStore.Contains(Key))
            {
                if (isolatedStore[Key] != value)
                {
                    isolatedStore[Key] = value;
                    valueChanged = true;
                }
            }
            else
            {
                isolatedStore.Add(Key, value);
                valueChanged = true;
            }

            return valueChanged;
        }


        // Get the current value of the setting, or if it is not found, set the setting to the default setting.
        private valueType GetValueOrDefault<valueType>(string Key, valueType defaultValue)
        {
            valueType value;

            if (isolatedStore.Contains(Key))
            {
                value = (valueType)isolatedStore[Key];
            }
            else
            {
                value = defaultValue;
            }

            return value;
        }


        private void Save()
        {
            isolatedStore.Save();
        }

        #endregion


        #region Properties

        public bool LocationSetting
        {
            get
            {
                return GetValueOrDefault<bool>(LocationSettingKeyName, LocationSettingDefault);
            }
            set
            {
                AddOrUpdateValue(LocationSettingKeyName, value);
                Save();
            }
        }

        public bool AutoRefreshSetting
        {
            get
            {
                return GetValueOrDefault<bool>(AutoRefreshSettingKeyName, AutoRefreshSettingDefault);
            }
            set
            {
                AddOrUpdateValue(AutoRefreshSettingKeyName, value);
                Save();
                if (value)
                {
                    App.Timer.Start();
                }
                else
                {
                    App.Timer.Stop();
                }
            }
        }

        #endregion
    }
}
