﻿using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Device.Location;
using System.Linq;

namespace SeaVan
{
    public class BorderCrossings : INotifyPropertyChanged
    {

        #region INotifyPropertyChanged

        public event PropertyChangedEventHandler PropertyChanged;
        private void NotifyPropertyChanged(string propertyName)
        {
            PropertyChangedEventHandler propertyChanged = PropertyChanged;
            if (null != propertyChanged)
            {
                propertyChanged(this, new PropertyChangedEventArgs(propertyName));
            }
        }

        #endregion


        #region Properties

        private ObservableCollection<BorderCrossing> crossings = new ObservableCollection<BorderCrossing>();
        public ObservableCollection<BorderCrossing> Crossings
        {
            get { return crossings; }
        }

        private string lastUpdateCa;
        public string LastUpdateCa
        {
            get { return lastUpdateCa; }
            set
            {
                if (lastUpdateCa != value)
                {
                    lastUpdateCa = value;
                    NotifyPropertyChanged("LastUpdateCa");
                }
            }
        }

        private string lastUpdateUs;
        public string LastUpdateUs
        {
            get { return lastUpdateUs; }
            set
            {
                if (lastUpdateUs != value)
                {
                    lastUpdateUs = value;
                    NotifyPropertyChanged("LastUpdateUs");
                }
            }
        }

        #endregion


        #region Init

        public BorderCrossings()
        {
            crossings.Add(new BorderCrossing
            {
                PortID = "300402",
                PortName = "Peace Arch / I5",
                CoordinatesCa = new GeoCoordinate(49.002315, -122.757277),
                CoordinatesUs = new GeoCoordinate(49.001851, -122.755826),
                ReadyDelayTimeUs = "-",
                CommercialDelayTimeUs = "-",
                FastDelayTimeUs = "-"
            });
            crossings.Add(new BorderCrossing 
            {   PortID = "300401", PortName = "Pacific Highway / SR543", 
                CoordinatesCa = new GeoCoordinate(49.003449,-122.735417),
                CoordinatesUs = new GeoCoordinate(49.001084,-122.735245),
                ReadyDelayTimeUs = "-"
            });
            crossings.Add(new BorderCrossing
            {
                PortID = "302301",
                PortName = "Aldegrove / Lynden",
                CoordinatesCa = new GeoCoordinate(49.002333, -122.485322),
                CoordinatesUs = new GeoCoordinate(49.002036, -122.484982),
                NexusDelayTimeUs = "-",
                ReadyDelayTimeUs = "-",
                CommercialDelayTimeUs = "-",
                FastDelayTimeUs = "-"
            });
            crossings.Add(new BorderCrossing 
            { 
                PortID = "300901", PortName = "Sumas / SR9",
                CoordinatesCa = new GeoCoordinate(49.002731, -122.265971),
                CoordinatesUs = new GeoCoordinate(49.002087, -122.265129),
                NexusDelayTimeUs = "-",
                ReadyDelayTimeUs = "-",
                CommercialDelayTimeUs = "-",
                FastDelayTimeUs = "-"
            });
        }

        #endregion


        #region Indexer

        public BorderCrossing this[string portID]
        {
            get
            {
                var b = from c in crossings where c.PortID == portID select c;
                if (b != null && b.Count() > 0)
                {
                    return b.First<BorderCrossing>();
                }
                return null;
            }
        }

        #endregion

    }
}

