//
//  SVAppDelegate.m
//  SeaVan
//
//  Created by Andrew Whitechapel on 4/16/12.
//  Copyright (c) 2012 __MyCompanyName__. All rights reserved.
//

#import "SVAppDelegate.h"
#import "SVContentController.h"
#import "SVBorderCrossings.h"
#import "SVURLConnectionWithTag.h"
#import "SVXMLParserUs.h"
#import "SVXMLParserCa.h"

static NSString *UrlCa = @"http://apps.cbp.gov/bwt/bwt.xml";
static NSString *UrlUs = @"http://wsdot.wa.gov/traffic/rssfeeds/CanadianBorderTrafficData/Default.aspx";
static NSUInteger kRefreshIntervalInSeconds = 180;

// We need a global dictionary to track the collection of NSURLConnection requests
// - or, more precisely, the collection of SVURLConnectionWithTag requests.
NSMutableDictionary *urlConnectionsByTag;

@interface SVAppDelegate (PrivateMethods)
- (void)fetchDataUs;
- (void)fetchDataCa;
@end

@implementation SVAppDelegate

@synthesize window = _window;
@synthesize contentController = _contentController;
@synthesize border;

- (BOOL)application:(UIApplication *)application 
didFinishLaunchingWithOptions:(NSDictionary *)launchOptions
{
    // Make sure the user settings are initialized.
    NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
    if (![defaults objectForKey:@"autorefresh"])
    {
        [defaults setBool:YES forKey:@"autorefresh"];
    }
    [defaults synchronize];
    
    // Fetch user settings to determine whether to start the auto-refresh timer.
    boolean_t isAutoRefreshOn = [defaults boolForKey:@"autorefresh"];
    if (isAutoRefreshOn)
    {
        [timer invalidate];
        timer = [NSTimer scheduledTimerWithTimeInterval:kRefreshIntervalInSeconds 
                                                 target:self 
                                               selector:@selector(onTimer) 
                                               userInfo:nil 
                                                repeats:YES];
    }

    // Initialize the main viewmodel.
    border = [[SVBorderCrossings alloc] init];
    
    self.window = [[UIWindow alloc] initWithFrame:[[UIScreen mainScreen] bounds]];
    // Override point for customization after application launch.
    
    [[NSBundle mainBundle] loadNibNamed:@"SVContentController" owner:self options:nil];

    [self.window addSubview:self.contentController.view];
    [self.window makeKeyAndVisible];
    
    // Initialize the connections dictionary.
    if (urlConnectionsByTag == nil) 
    {
        urlConnectionsByTag = [[NSMutableDictionary alloc] init];
    }

    return YES;
}

- (void)applicationWillResignActive:(UIApplication *)application
{
    // Sent when the application is about to move from active to inactive state. This can occur for certain types of temporary interruptions (such as an incoming phone call or SMS message) or when the user quits the application and it begins the transition to the background state.
    // Use this method to pause ongoing tasks, disable timers, and throttle down OpenGL ES frame rates. Games should use this method to pause the game.
}

- (void)applicationDidEnterBackground:(UIApplication *)application
{
    // Use this method to release shared resources, save user data, invalidate timers, and store enough application state information to restore your application to its current state in case it is terminated later. 
    // If your application supports background execution, this method is called instead of applicationWillTerminate: when the user quits.
}

- (void)applicationWillEnterForeground:(UIApplication *)application
{
    // Called as part of the transition from the background to the inactive state; here you can undo many of the changes made on entering the background.
}

- (void)applicationDidBecomeActive:(UIApplication *)application
{
    // Restart any tasks that were paused (or not yet started) while the application was inactive. If the application was previously in the background, optionally refresh the user interface.
}

- (void)applicationWillTerminate:(UIApplication *)application
{
    // Called when the application is about to terminate. Save data if appropriate. See also applicationDidEnterBackground:.
}

// Url Request methods /////////////////////////////////////////////////////////

- (void)onTimer
{
    [self refreshData];
}

- (void)refreshData
{
    [self fetchDataUs];
    [self fetchDataCa];
}

- (void)fetchDataUs
{
    rawDataUs = [[NSMutableData alloc] init];
    NSURL *url = [NSURL URLWithString:UrlUs];
    NSURLRequest *request = [NSURLRequest requestWithURL:url];    
    connectionUs = 
    [[SVURLConnectionWithTag alloc] initWithRequest:request 
                                           delegate:self 
                                   startImmediately:YES 
                                                tag:[NSNumber numberWithInt:ConnectionUs]];
}

- (void)fetchDataCa
{
    rawDataCa = [[NSMutableData alloc] init];
    NSURL *url = [NSURL URLWithString:UrlCa];
    NSURLRequest *request = [NSURLRequest requestWithURL:url];    
    connectionCa = 
    [[SVURLConnectionWithTag alloc] initWithRequest:request 
                                           delegate:self 
                                   startImmediately:YES 
                                                tag:[NSNumber numberWithInt:ConnectionCa]];
}

// The delegates for the URL connections must specify the custom subclass as the connection type.
- (void)connection:(SVURLConnectionWithTag *)connection didReceiveData:(NSData *)data
{
    // If this data is for a connection that's not already in our collection, add it now.
    if ([urlConnectionsByTag objectForKey:connection.tag] == nil) 
    {
        NSMutableData *newData = [[NSMutableData alloc] initWithData:data];
        [urlConnectionsByTag setObject:newData forKey:connection.tag];
        return;
    } 
    else 
    {
        // Append the incoming chunk of data to the NSMutableData container identified 
        // by the connection tag. The data always comes in in the correct order.
        [[urlConnectionsByTag objectForKey:connection.tag] appendData:data];
    }
}

- (void)connectionDidFinishLoading:(SVURLConnectionWithTag *)connection 
{
//    NSString *xmlReceived = 
//    [[NSString alloc] initWithData:[urlConnectionsByTag objectForKey:connection.tag] 
//                          encoding:NSUTF8StringEncoding];
    
    //NSLog(@"xmlReceived (Us) = %@", xmlReceived);
    NSXMLParser *parser = 
    [[NSXMLParser alloc] initWithData:[urlConnectionsByTag objectForKey:connection.tag]];
    [parser setDelegate:self];
    
    // Hmm... parse is a blocking call.
    [parser parse]; 
    
    // Parse the data we've received, and build object trees for the view fields.
    switch ([connection.tag intValue]) 
    {
        case ConnectionUs :
            // Use either message or dot notation.
            //border.lastUpdateUs = [xmlParserUs lastUpdate];
            border.lastUpdateUs = xmlParserUs.lastUpdate;
            //NSLog(@"%@", border.lastUpdateUs);
            break;
        case ConnectionCa :
            border.lastUpdateCa = [xmlParserCa lastUpdate];
            //NSLog(@"received Ca data: %@", border.lastUpdateCa);
            break;
        default:
            break;
    }
    
    [_contentController updateViewFromData];
    
    // We're done with this connection, so remove it from the collection: we need
    // to do this in both the connectionDidFinishLoading and in the didFailWithError.
    [urlConnectionsByTag removeObjectForKey:connection.tag];
    //xmlReceived = nil;
    connection = nil;
}

- (void)connection:(SVURLConnectionWithTag *)connection didFailWithError:(NSError *)error
{
    connectionUs = nil;
    rawDataUs = nil;
    connectionCa = nil;
    rawDataCa = nil;
    
    NSString *errorString = 
    [NSString stringWithFormat:@"cannot get data: @%", [error localizedDescription]];
    UIAlertView *alert = 
    [[UIAlertView alloc] initWithTitle:@"Error" 
                               message:errorString delegate:nil cancelButtonTitle:@"OK" otherButtonTitles:nil];
    [alert show];
    
    // We're done with this connection, so remove it from the collection.
    [urlConnectionsByTag removeObjectForKey:connection.tag];
}


// XML Parsing /////////////////////////////////////////////////////////////////

- (void)parser:(NSXMLParser *)parser
didStartElement:(NSString *)elementName 
  namespaceURI:(NSString *)namespaceURI 
 qualifiedName:(NSString *)qName 
    attributes:(NSDictionary *)attributeDict
{
    //NSLog(@"%@ found a %@ element", self, elementName);
    if ([elementName isEqual:@"rss"]) 
    {
        xmlParserUs = [[SVXMLParserUs alloc] init];
        [xmlParserUs setParentParserDelegate:self];
        [parser setDelegate:xmlParserUs];
    }
    else if ([elementName isEqual:@"border_wait_time"])
    {
        xmlParserCa = [[SVXMLParserCa alloc] init];
        [xmlParserCa setParentParserDelegate:self];
        [parser setDelegate:xmlParserCa];
    }
}



@end
