//
//  SVXMLParserCa.m
//  SeaVan
//
//  Created by Andrew Whitechapel on 4/20/12.
//  Copyright (c) 2012 __MyCompanyName__. All rights reserved.
//

#import "SVXMLParserCa.h"
#import "SVBorderCrossing.h"
#import "SVXMLParserCaPort.h"
#import "constants.h"

static NSUInteger kCrossingCount = 4;

@implementation SVXMLParserCa

@synthesize parentParserDelegate, lastUpdate, portElements;

// Some syntax errors will not even produce IDE nor compile-time warnings, because
// they're innocuous, but they will have bizarre side-effects on the editor (such
// as failing to indent correctly in the file that contains the error).
//@interface SVXMLParserCa (PrivateMethods)
////- (NSString *)getValueOrDefault:(NSDictionary *)dictionary key:(NSString *)key;
//@end

- (id) init
{
    if (self = [super init])
    {
        portElements = [[NSMutableArray alloc] init];
        portCount = 0;

        bcPorts = [[NSMutableArray alloc] initWithCapacity:appDelegate.border.crossings.count];
        for (SVBorderCrossing *crossing in appDelegate.border.crossings)
        {
            [bcPorts addObject:crossing.portID];
        }
    }
    return self;
}

- (void)parser:(NSXMLParser *)parser 
    didStartElement:(NSString *)elementName 
    namespaceURI:(NSString *)namespaceURI 
    qualifiedName:(NSString *)qualifiedName 
    attributes:(NSDictionary *)attributeDict
{
    //NSLog(@"\t%@ found a %@ element", self, elementName);
    if ([elementName isEqual:@"last_updated_date"]) 
    {
        currentString = [[NSMutableString alloc] init];
        updateDate = currentString;
    }
    if ([elementName isEqual:@"last_updated_time"]) 
    {
        currentString = [[NSMutableString alloc] init];
        updateTime = currentString;
    }
    else if ([elementName isEqual:@"port_number"]) 
    {
        currentString = [[NSMutableString alloc] init];
        currentPort = currentString;
    }
}

- (void)parser:(NSXMLParser *)parser foundCharacters:(NSString *)str
{
    [currentString appendString:str];
}

- (void)parser:(NSXMLParser *)parser 
 didEndElement:(NSString *)elementName 
  namespaceURI:(NSString *)namespaceURI 
 qualifiedName:(NSString *)qName
{
    // Time is after date in the XML, so we wait until we hit the end of the time element
    // before concatenating date + time.
    if ([elementName isEqual:@"last_updated_time"]) 
    {
        @try 
        {
            // Use a date formatter to read this string into a date, and then write
            // it out again in the target format. The original time was 5 hrs behind
            // GMT, and we want the final time in PST/PDT, which is 8 hrs behind GMT.
            NSString *updateString = [NSString stringWithFormat:@"%@ %@", updateDate, updateTime];
            NSDateFormatter *df = [[NSDateFormatter alloc] init];
            [df setDateFormat:@"yyyy-M-d HH:mm:ss"];
            [df setTimeZone:[NSTimeZone timeZoneForSecondsFromGMT:(-5 * 3600)]];
            NSDate *date = [df dateFromString:updateString];
            [df setDateFormat:@"dd/MM/yyyy HH:mm:ss"];
            [df setTimeZone:[NSTimeZone timeZoneForSecondsFromGMT:(-8 * 3600)]];
            updateString = [df stringFromDate:date];
            //NSLog(@"%@ =  %@", date, updateString);
            // "30/03/2012 14:15:22".
            
            lastUpdate  = [NSString stringWithFormat:@" last update (van): %@", updateString];
        }
        @catch (NSException* exception)
        {
            NSLog(@"SVXMLParserCa.parser: Caught %@: %@", [exception name], [exception reason]);
            UIAlertView* alert = [[UIAlertView alloc] initWithTitle:@"XML Parse Error" 
                                                            message:@"Error reading Canadian border data" 
                                                           delegate:nil 
                                                  cancelButtonTitle:@"OK" 
                                                  otherButtonTitles:nil];
            [alert show];
        }                
    }
    else if ([elementName isEqual:@"port_number"])
    {
        if ([bcPorts containsObject:currentPort])
        {
            @try
            {
                // In this parser, we read through to the port_number, and only delegate
                // to our child parser when we find one of the 4 BC ports. This way, 
                // the child parser will only ever get the inner port data for one of 
                // the 4 BC ports.
                
                // When we find a port entry, we instantiate the child parser that will
                // handle the port, set its parent parser to point back to us, and set
                // it as the current parser. We pass it the corresponding crossing object
                // so that it can set the crossing field values directly.
                SVBorderCrossing *currentCrossing;
                for (SVBorderCrossing *crossing in appDelegate.border.crossings)
                {
                    if ([crossing.portID isEqual:currentPort])
                    {
                        currentCrossing = crossing;
                        break;
                    }
                }
                
                SVXMLParserCaPort *port = 
                [[SVXMLParserCaPort alloc] initWithCrossing:currentCrossing];
                [port setParentParserDelegate:self];
                [parser setDelegate:port];
                
                // We need to keep a reference to each child parser we create, otherwise
                // it gets deallocated, and we need to query it later.
                [portElements addObject:port];
                portCount++;
            }
            @catch (NSException* exception)
            {
                NSLog(@"SVXMLParserCa.parser: Caught %@: %@", [exception name], [exception reason]);
                UIAlertView* alert = [[UIAlertView alloc] initWithTitle:@"XML Parse Error" 
                                                                message:@"Error reading Canadian border data" 
                                                               delegate:nil 
                                                      cancelButtonTitle:@"OK" 
                                                      otherButtonTitles:nil];
                [alert show];
            }                
        }
    }
        
    currentString = nil;
    
    // If we've completed 4 ports, we can stop.
    if (portCount > kCrossingCount)
    {
        [parser setDelegate:parentParserDelegate];
    }
}

@end
