//
//  SVXMLParserCaPort.m
//  SeaVan
//
//  Created by Andrew Whitechapel on 4/21/12.
//  Copyright (c) 2012 __MyCompanyName__. All rights reserved.
//

#import "SVXMLParserCaPort.h"
#import "SVBorderCrossing.h"

@implementation SVXMLParserCaPort

@synthesize parentParserDelegate;

- (id) initWithCrossing:(SVBorderCrossing *)crossing
{
    if (self = [super init])
    {
        crossingType = CrossingTypeUnknown;
        laneType = LaneTypeUnknown;
        currentCrossing = crossing;
    }
    return self;
}

- (void)parser:(NSXMLParser *)parser 
didStartElement:(NSString *)elementName 
  namespaceURI:(NSString *)namespaceURI 
 qualifiedName:(NSString *)qualifiedName 
    attributes:(NSDictionary *)attributeDict
{
    //NSLog(@"\t%@ found a %@ element", self, elementName);
    
    // The only data value we want to retrieve is the wait time, but we need to
    // track other entries so that we can tell which specific crossing type and
    // lane type the wait time is for. We also need the operational status data
    // to fine-tune the wait information string.
    if ([elementName isEqual:@"commercial_vehicle_lanes"])
    {
        crossingType = CrossingTypeTruck;
    }
    else if ([elementName isEqual:@"passenger_vehicle_lanes"])
    {
        crossingType = CrossingTypeCar;
    }
    else if ([elementName isEqual:@"standard_lanes"])
    {
        laneType = LaneTypeStandard;
    }
    else if ([elementName isEqual:@"FAST_lanes"])
    {
        laneType = LaneTypeFast;
    }
    else if ([elementName isEqual:@"NEXUS_SENTRI_lanes"])
    {
        laneType = LaneTypeNexus;
    }
    else if ([elementName isEqual:@"ready_lanes"])
    {
        laneType = LaneTypeReady;
    }
    else if ([elementName isEqual:@"operational_status"])
    {
        currentString = [[NSMutableString alloc] init];
        operationalStatus = currentString;
    }
    else if ([elementName isEqual:@"delay_minutes"])
    {
        currentString = [[NSMutableString alloc] init];
        delayTime = currentString;
    }
}

- (void)parser:(NSXMLParser *)parser foundCharacters:(NSString *)str
{
    [currentString appendString:str];
}

- (void)parser:(NSXMLParser *)parser 
 didEndElement:(NSString *)elementName 
  namespaceURI:(NSString *)namespaceURI 
 qualifiedName:(NSString *)qName
{        
    if ([elementName isEqual:@"delay_minutes"])
    {
        // Massage the raw wait time strings, and then set the clean
        // strings into the current crossing object.
        NSString *waitTime = [delayTime stringByTrimmingCharactersInSet:
                            [NSCharacterSet whitespaceCharacterSet]];
        if (!waitTime || waitTime.length == 0)
        {
            waitTime = @"no data";
        }
        NSString *status = [operationalStatus stringByTrimmingCharactersInSet:
                            [NSCharacterSet whitespaceCharacterSet]];
        status = [status lowercaseString];
        if ([status isEqual:@"delay"])
        {
            waitTime = [waitTime stringByReplacingOccurrencesOfString:@"min" withString:@""];
            waitTime = [waitTime stringByTrimmingCharactersInSet:
                        [NSCharacterSet whitespaceCharacterSet]];
            if (!waitTime || waitTime.length == 0)
            {
                waitTime = @"delay";
            }
        }
        else if ([status isEqual:@"no delay"])
        {
            waitTime = @"0";
        }
        else if ([status isEqual:@"n/a"])
        {
            waitTime = @"-";
        }
        else if ([status isEqual:@"lanes closed"])
        {
            waitTime = @"closed";
        }
        else if ([status isEqual:@"update pending"])
        {
            waitTime = @"updating";
        }
        else 
        {
            waitTime = status;
        }
        
        if (crossingType == CrossingTypeTruck)
        {
            if (laneType == LaneTypeStandard)
            {
                currentCrossing.commercialDelayTimeCa = waitTime;
            }
            else if (laneType == LaneTypeFast)
            {
                currentCrossing.fastDelayTimeCa = waitTime;
            }
        }
        else if (crossingType == CrossingTypeCar)
        {
            if (laneType == LaneTypeStandard)
            {
                currentCrossing.passengerDelayTimeCa = waitTime;
            }
            else if (laneType == LaneTypeNexus)
            {
                currentCrossing.nexusDelayTimeCa = waitTime;
            }
            else if (laneType == LaneTypeReady)
            {
                currentCrossing.readyDelayTimeCa = waitTime;
            }
        }
    }
    
    currentString = nil;

    // Stop when we get to the end of the passenger lanes, and revert to our
    // parent parser.
    if ([elementName isEqual:@"passenger_vehicle_lanes"])
    {        
        [parser setDelegate:parentParserDelegate];
    }
}

@end
