//
//  SVXMLParserUs.m
//  SeaVan
//
//  Created by Andrew Whitechapel on 4/20/12.
//  Copyright (c) 2012 __MyCompanyName__. All rights reserved.
//

#import "SVXMLParserUs.h"
#import "SVBorderCrossing.h"
#import "constants.h"

@implementation SVXMLParserUs

@synthesize parentParserDelegate, lastUpdate;

@interface SVXMLParserUs (PrivateMethods)
- (NSString *)getValueOrDefault:(NSDictionary *)dictionary key:(NSString *)key;
@end

- (void)parser:(NSXMLParser *)parser 
didStartElement:(NSString *)elementName 
  namespaceURI:(NSString *)namespaceURI 
 qualifiedName:(NSString *)qualifiedName 
    attributes:(NSDictionary *)attributeDict
{
    //NSLog(@"\t%@ found a %@ element", self, elementName);
    if ([elementName isEqual:@"title"]) 
    {
        currentString = [[NSMutableString alloc] init];
        [self setLastUpdate:currentString];
    }
}

- (void)parser:(NSXMLParser *)parser foundCharacters:(NSString *)str
{
    [currentString appendString:str];
}

- (void)parser:(NSXMLParser *)parser foundCDATA:(NSData *)CDATABlock
{
    NSString *cdataString = 
        [[NSString alloc] initWithData:CDATABlock encoding:NSUTF8StringEncoding];
    //NSLog(@"%@ found a CDATABlock: %@", self, cdataString);
    
    cdataString = [cdataString lowercaseString];
    cdataString = [cdataString stringByReplacingOccurrencesOfString:@"<br/>" withString:@""];
    cdataString = [cdataString stringByReplacingOccurrencesOfString:@"<strong>station name:</strong>" withString:@","];
    cdataString = [cdataString stringByReplacingOccurrencesOfString:@"<strong>wait time:</strong>" withString:@"|"];
    
    NSArray *crossingData = [cdataString componentsSeparatedByString:@","];
    NSMutableDictionary *usCrossings = [[NSMutableDictionary alloc] init];
    for (int i = 1; i < [crossingData count]; i++)
    {
        NSArray *crossingParts = [[crossingData objectAtIndex:i]componentsSeparatedByString:@"|"];
        [usCrossings setObject:[crossingParts objectAtIndex:1] forKey:[crossingParts objectAtIndex:0]];
    }

    for (SVBorderCrossing *crossing in appDelegate.border.crossings)
    {
        int portID = [crossing.portID intValue];
        switch (portID)
        {
            case 300401:
                crossing.PassengerDelayTimeUs = [self getValueOrDefault:usCrossings key:@"sr543"];
                crossing.NexusDelayTimeUs = [self getValueOrDefault:usCrossings key:@"sr543nexus"];
                crossing.CommercialDelayTimeUs = [self getValueOrDefault:usCrossings key:@"sr543trucks"];
                crossing.FastDelayTimeUs = [self getValueOrDefault:usCrossings key:@"sr543trucksfast"];
                break;
            case 300402:
                crossing.PassengerDelayTimeUs = [self getValueOrDefault:usCrossings key:@"i5"];
                crossing.NexusDelayTimeUs = [self getValueOrDefault:usCrossings key:@"i5nexus"];
                break;
            case 300901:
                crossing.PassengerDelayTimeUs = [self getValueOrDefault:usCrossings key:@"sr9"];
                break;
            case 302301:
                crossing.PassengerDelayTimeUs = [self getValueOrDefault:usCrossings key:@"sr539"];
                break;
            default:
                break;
        }
    }
}

- (NSString *)getValueOrDefault:(NSDictionary *)dictionary key:(NSString *)key
{
    NSString *s = [dictionary objectForKey:key];
    if (!s)
    {
        s = @"no data";
    }
    return s;
}

- (void)parser:(NSXMLParser *)parser 
 didEndElement:(NSString *)elementName 
  namespaceURI:(NSString *)namespaceURI 
 qualifiedName:(NSString *)qName
{
    if ([elementName isEqual:@"title"])
    {
        if (lastUpdate)
        {
            // There are 2 title elements: we want the one that's a child of the item element,
            // which has the given prefix.
            NSRange range = [lastUpdate rangeOfString:@"Canadian Border Crossings - " 
                                              options:NSCaseInsensitiveSearch];
            if (range.location != NSNotFound) 
            {
                @try
                {
                    // "Canadian Border Crossings - Fri Jun 15 11:15:00 PDT 2012"
                    lastUpdate = [lastUpdate stringByReplacingOccurrencesOfString:@"Canadian Border Crossings - " withString:@""];
                    // "Fri Jun 15 11:15:00 PDT 2012"

                    // If you try to just use integer array subscripts for NSArray objects, you'll get the fairly self-explanatory error message: "subscript requires size of interface NSArray which is not constant in non-stable ABI." This is because array subscripts use pointer arithmetic, but NSArray is an Objective C interface, and the compiler can't be sure what the size of the type is. The correct way to index into an NSArray is to send the object a message asking for the item at a given index.
                    NSArray *dateParts = [lastUpdate componentsSeparatedByString:@" "];
                    NSString *updateString = 
                        [NSString stringWithFormat:@"%@, %@ %@ %@ %@", 
                        [dateParts objectAtIndex:0], [dateParts objectAtIndex:2], 
                        [dateParts objectAtIndex:1], [dateParts objectAtIndex:5], [dateParts objectAtIndex:3]];
                    // "Fri, 30 Mar 2012 14:15:22".
                    
                    // Use a date formatter to read this string into a date, and then write
                    // it out again in the target format.
                    NSDateFormatter *df = [[NSDateFormatter alloc] init];
                    [df setDateFormat:@"E, d M y H:m:s"];
                    NSDate *date = [df dateFromString:updateString];
                    [df setDateFormat:@"dd/MM/yyyy HH:mm:ss"];
                    updateString = [df stringFromDate:date];
                    //NSLog(@"%@ =  %@", date, updateString);
                    // "30/03/2012 14:15:22".

                    lastUpdate  = [NSString stringWithFormat:@" last update (sea): %@", updateString];
                }
                @catch (NSException* exception)
                {
                    NSLog(@"SVXMLParserUs.parser: Caught %@: %@", [exception name], [exception reason]);
                    UIAlertView* alert = [[UIAlertView alloc] initWithTitle:@"XML Parse Error" 
                                                                    message:@"Error reading US border data" 
                                                                   delegate:nil 
                                                          cancelButtonTitle:@"OK" 
                                                          otherButtonTitles:nil];
                    [alert show];
                }                
            }
        }
    }

    // If we were in an element that we were collecting the string for, this
    // appropriately releases our hold on it and the permanent ivar keeps ownership
    // of it. If we weren't parsing such an element, currentString is nil already.
    currentString = nil;
    
    // If the element that ended was the outermost one we care about, then we give
    // up control to whoever gave us control in the first place.
    if ([elementName isEqual:@"rss"])
    {
        [parser setDelegate:parentParserDelegate];
    }
}

@end

